require 'test_helper'
require 'gradesfirst/cli'

describe GradesFirst::CLI do

  def assert_command_hooked_up(shell_command, klass)
    assert_includes(
      klass.instance_methods,
      :execute,
      "#{klass.name} does not implement the #execute method"
    )
    assert_includes(
      klass.instance_methods,
      :response,
      "#{klass.name} does not implement the #response method"
    )

    execute_arity = klass.instance_method(:execute).arity
    if execute_arity < 0
      execute_arity += 1
      execute_arity = execute_arity.abs
    end
    expected_execute_parameters = Array.new(execute_arity, Object)

    expected_output = 'Expected Response'
    command_instance = Minitest::Mock.new.
      expect(:execute, nil, expected_execute_parameters).
      expect(:response, expected_output)

    stdout, stderr = capture_io do
      klass.stub :new, command_instance do
        GradesFirst::CLI.start Shellwords.split(shell_command)
      end
    end

    assert_equal '', stderr, "Error reported to standard error by CLI command \"#{shell_command}\""
    assert_match expected_output, stdout, "Incorrect output from CLI command \"#{shell_command}\""
    command_instance.verify
  end

  specify 'branch command is hooked up' do
    assert_command_hooked_up 'branch', GradesFirst::BranchCommand
  end

  specify 'codetalk command is hooked up' do
    assert_command_hooked_up 'codetalk', GradesFirst::CodeTalkCommand
  end

  specify 'commit-message command is hooked up' do
    assert_command_hooked_up 'commit-message', GradesFirst::CommitMessageCommand
  end

  specify 'pair command is hooked up' do
    assert_command_hooked_up 'pair', GradesFirst::PairCommand
  end

  # Waiting on fix for subcommands with default command.
  # https://github.com/erikhuda/thor/pull/374
  #specify 'task command is hooked up to default list subcommand' do
  #  assert_command_hooked_up 'task', GradesFirst::TaskListCommand
  #end

  specify 'task add command is hooked up' do
    assert_command_hooked_up(
      'task add "This is a task."',
      GradesFirst::TaskAddCommand
    )
  end

  specify 'task delete command is hooked up' do
    assert_command_hooked_up('task delete 1', GradesFirst::TaskDeleteCommand)
  end

  specify 'task list command is hooked up' do
    assert_command_hooked_up 'task list', GradesFirst::TaskListCommand
  end

  specify 'task move command is hooked up' do
    assert_command_hooked_up('task move 5 1', GradesFirst::TaskMoveCommand)
  end

  specify 'task toggle command is hooked up' do
    assert_command_hooked_up('task toggle 1', GradesFirst::TaskToggleCommand)
  end

end
