require 'test_helper'
require 'gradesfirst/code_talk_command'

describe GradesFirst::CodeTalkCommand do
  before do
    @command = GradesFirst::CodeTalkCommand.new
    @code_talkers = [
      'Mickey Mouse',
      'Cat Woman',
      'Wonder Woman',
      'Superman',
      'Minnie Mouse',
      'Ironman'
    ]
  end

  specify '#execute schedules code talks correctly' do
    config = OpenStruct.new(
      code_talkers: @code_talkers,
      day_zero: Time.parse("2014-3-10 00:00:00")
    )
    @command.stub :config, config do
      @command.stub :today, Date.parse("2014-3-25") do
        @command.execute
        assert_equal(
          fixture_file('code_talk.txt'),
          @command.response,
          'Code talk schedule is not correct.'
        )
      end
    end
  end

  def assert_talk_count(date, count, message)
    @command.stub :today, Date.parse(date) do
      assert_equal(count, @command.send(:talk_count), message)
    end
  end

  specify '#talk_count counts talks correctly' do
    assert_talk_count("2014-3-10", 0, 'Zero day should not count any talks')
    assert_talk_count("2014-3-11", 0, 'Tuesday should not be counted')
    assert_talk_count("2014-3-12", 0, 'Wednesday should not be counted')
    assert_talk_count("2014-3-13", 0, 'Thursday should not be counted')
    assert_talk_count("2014-3-14", 1, 'Thursday should be counted on Friday')
    assert_talk_count("2014-3-25", 2, 'Test date count is wrong')
  end
end
