require 'test_helper'
require 'pivotal_tracker'
require 'gradesfirst/commit_message_command'
require 'pivotal_tracker'

describe GradesFirst::CommitMessageCommand do
  describe '#response' do
    it 'responds with formatted commit message' do
      command = GradesFirst::CommitMessageCommand.new
      expect_requests stub_bar_story_request do
        command.stub :current_branch, "test/#{bar_story_id}", &call_execute
      end
      assert_equal fixture_file('commit_message.txt'), command.response
    end
  end

  describe '#constrain_line_length' do
    def test_string
      'Lorem ipsum dolor sit amet, consectetur adipiscing elit. Fusce vitae.'
    end

    def test_string_ending_with_a_letter
      'Lorem ipsum dolor sit amet'
    end

    def test_string_with_line_feeds_at_30
      "Lorem ipsum dolor sit amet,\nconsectetur adipiscing elit.\nFusce vitae."
    end

    it 'adds new lines at natural breaks when length is exceeded' do
      command = GradesFirst::CommitMessageCommand.new
      assert_equal(
        test_string_with_line_feeds_at_30,
        command.send(:constrain_line_length, test_string, 30)
      )
    end

    it 'does nothing when length is not exceeded' do
      command = GradesFirst::CommitMessageCommand.new
      assert_equal(
        test_string,
        command.send(:constrain_line_length, test_string, 80)
      )
    end

    it 'does not wrap when ending with a letter' do
      command = GradesFirst::CommitMessageCommand.new
      assert_equal(
        test_string_ending_with_a_letter,
        command.send(:constrain_line_length, test_string_ending_with_a_letter, 80)
      )
    end
  end
end
