module GradesFirst
  module PivotalTestHelper

    def bar_story_id
      '29384793'
    end

    def foo_story_id
      '57348714'
    end

    def missing_story_id
      '29384799'
    end

    def pivotal_tracker_setup
      PivotalTracker.api_token = 'test_token'
    end

    def stub_bar_story_request
      stub_pivotal_request(
        :get,
        'stories/' + bar_story_id,
        response: { body: fixture_file('story_bar.json') }
      )
    end

    def stub_bar_story_tasks_request
      stub_pivotal_request(
        :get,
        urn_for_bar_story_tasks,
        response: { body: fixture_file('tasks.json') }
      )
    end

    def stub_foo_story_request
      stub_pivotal_request(
        :get,
        'stories/' + foo_story_id,
        response: { body: fixture_file('story_foo.json') }
      )
    end

    def stub_missing_story_request
      stub_pivotal_request(
        :get,
        'stories/' + missing_story_id,
        response: { status: 404 }
      )
    end

    # Simplify and DRY up stubbing out of PivotalTracker requests.
    def stub_pivotal_request(method, urn, options = {})
      default_request = {
        headers: { 'X-TrackerToken' => 'test_token' }
      }
      if method == :post
        default_request[:headers]['Content-Type'] = 'application/json'
      end
      default_response = {
        headers: { 'Content-Type' => 'application/json' },
        body: '',
        status: 200
      }

      uri = "https://www.pivotaltracker.com:443/services/v5/#{urn}"
      stub_request(method, uri).
        with(default_request.merge(options[:request] || {})).
        to_return(default_response.merge(options[:response] || {}))

      RequestExpectation.new(method, uri)
    end

    def urn_for_bar_story_tasks
      "projects/10687/stories/#{bar_story_id}/tasks"
    end

    def urn_for_bar_story_task(task_id)
      urn_for_bar_story_tasks + "/#{task_id}"
    end

    module ClassMethods

      def specify_response_for_invalid_story(*args)
        specify '#response responds with error message with an invalid story' do
          expect_requests stub_missing_story_request do
            @command.stub :current_branch, missing_story_id do
              @command.execute(*args)
            end
          end
          assert_equal @command.send(:story_error_message), @command.response
        end
      end

      def specify_response_for_invalid_task_position(method, *args)
        specify '#response when position does not exist' do
          stub_bar_story_request
          stub_bar_story_tasks_request
          @command.stub :current_branch, bar_story_id do
            @command.execute(*args)
          end
          assert_equal @command.send(:position_invalid_message), @command.response
          assert_not_requested method, /.*/
        end
      end

    end

  end
end
