# This class encapsulates creating http request expectations with the webmock
# assert_requested assertion.
#
# Example:
#   expectation = RequestExpectation(:get, 'http://example.com').times(2)
#   assert_requested(*expectation.to_args)
class RequestExpectation
  # Store the HTTP request method and uri that are expected and specifies the
  # default number of times that the request is expected to made.
  def initialize(method, uri)
    @method = method
    @uri = uri
    @times = 1
  end

  # Specifies the number of times that a request is expected to be made.
  #
  # Example:
  #   RequestExpectation.new(:get, 'http://example.com').times(2).to_args
  #
  # Retuns an instance of itself to allow for method chaining.
  def times(value)
    @times = value
    self
  end

  # Creates a set of method arguments for the Webmock assert_requested method.
  #
  # Returns an array of Webmock assert_request method arguments.
  def to_args
    [@method, @uri, { times: @times }]
  end
end
