require 'test_helper'
require 'gradesfirst/task_add_command'

describe GradesFirst::TaskAddCommand do
  before do
    @task_description = 'New task.'
    @command = GradesFirst::TaskAddCommand.new
  end

  def stub_bar_story_task_add_request(response)
    stub_pivotal_request(
      :post,
      urn_for_bar_story_tasks,
      request: { body: "{\"description\":\"#{@task_description}\"}" },
      response: response
    )
  end

  def stub_failing_bar_story_task_add_request
    stub_bar_story_task_add_request status: 500
  end

  def stub_working_bar_story_task_add_request
    stub_bar_story_task_add_request body: fixture_file('task.json')
  end

  specify '#response responds with confirmation and task list' do
    stubbed_requests = [
      stub_bar_story_request,
      stub_bar_story_tasks_request,
      stub_working_bar_story_task_add_request
    ]

    response = nil
    expect_requests stubbed_requests do
      @command.stub :current_branch, bar_story_id do
        @command.execute(@task_description)
      end
      response = @command.response
    end
    assert_equal fixture_file('task_added.txt'), response
  end

  specify '#response when adding the task fails' do
    stub_bar_story_request
    stub_failing_bar_story_task_add_request
    @command.stub :current_branch, bar_story_id do
      @command.execute(@task_description)
    end
    assert_equal @command.send(:task_error_message), @command.response
  end

  specify_response_for_invalid_story @task_description
end
