require 'test_helper'
require 'gradesfirst/task_move_command'

describe GradesFirst::TaskMoveCommand do
  before do
    @task_id = 6
    @from_position = '2'
    @to_position = '1'
    @command = GradesFirst::TaskMoveCommand.new
  end

  def stub_bar_story_task_move_request(task_id, to_position, response)
    stub_pivotal_request(
      :put,
      urn_for_bar_story_task(task_id),
      request: { body: "{\"position\":#{to_position}}" },
      response: response
    )
  end

  def stub_failing_bar_story_task_move_request(task_id, to_position)
    stub_bar_story_task_move_request(
      task_id,
      to_position,
      status: 500
    )
  end

  def stub_working_bar_story_task_move_request(task_id, to_position)
    stub_bar_story_task_move_request(
      task_id,
      to_position,
      body: fixture_file('task.json')
    )
  end

  specify '#execute moves the task to a new position' do
    stubbed_requests = [
      stub_bar_story_request,
      stub_bar_story_tasks_request.times(2),
      stub_working_bar_story_task_move_request(@task_id, @to_position)
    ]

    response = nil
    expect_requests stubbed_requests do
      @command.stub :current_branch, bar_story_id do
        @command.execute(@from_position, @to_position)
      end
      response = @command.response
    end
    assert_equal fixture_file('task_moved.txt'), response
  end

  specify '#response when moving the task fails' do
    stub_bar_story_request
    stub_bar_story_tasks_request
    stub_failing_bar_story_task_move_request(@task_id, @to_position)
    @command.stub :current_branch, bar_story_id do
      @command.execute(@from_position, @to_position)
    end
    assert_equal @command.send(:task_error_message), @command.response
  end

  specify_response_for_invalid_task_position :put, @from_position, @to_position
  specify_response_for_invalid_story @from_position, @to_position
end
