require 'test_helper'
require 'gradesfirst/task_toggle_command'

describe GradesFirst::TaskToggleCommand do
  before do
    @task_position = '2'
    @invalid_position = '8'
    @task_id = 6
    @command = GradesFirst::TaskToggleCommand.new
  end

  def stub_bar_story_task_toggle_request(task_id, response)
    stub_pivotal_request(
      :put,
      urn_for_bar_story_task(task_id),
      request: { body: "{\"complete\":true}" },
      response: response
    )
  end

  def stub_failing_bar_story_task_toggle_request(task_id)
    stub_bar_story_task_toggle_request(task_id, status: 500)
  end

  def stub_working_bar_story_task_toggle_request(task_id)
    stub_bar_story_task_toggle_request(task_id, body: fixture_file('task.json'))
  end

  specify '#execute toggles completion status of the task' do
    stubbed_requests = [
      stub_bar_story_request,
      stub_bar_story_tasks_request.times(2),
      stub_working_bar_story_task_toggle_request(@task_id)
    ]

    response = nil
    expect_requests stubbed_requests do
      @command.stub :current_branch, bar_story_id do
        @command.execute(@task_position)
      end
      response = @command.response
    end
    assert_equal fixture_file('task_toggled.txt'), response
  end

  specify '#response when toggling the task fails' do
    stub_bar_story_request
    stub_bar_story_tasks_request
    stub_failing_bar_story_task_toggle_request(@task_id)
    @command.stub :current_branch, bar_story_id do
      @command.execute(@task_position)
    end
    assert_equal @command.send(:task_error_message), @command.response
  end

  specify_response_for_invalid_task_position :put, @invalid_position
  specify_response_for_invalid_story @task_position
end
