$LOAD_PATH.unshift(File.expand_path('../../lib', __FILE__))

require 'rubygems'
gem 'minitest' if RUBY_VERSION > '1.9'
require 'minitest/autorun'
require 'webmock/minitest'
require 'pry'

Dir["#{File.dirname(__FILE__)}/support/**/*.rb"].each { |f| require f }

class MiniTest::Unit::TestCase
  include GradesFirst::PivotalTestHelper
  extend GradesFirst::PivotalTestHelper::ClassMethods

  def setup
    pivotal_tracker_setup
    GradesFirst::Config.load_paths = ["test/fixtures/.env.gf"]
  end

  # Retreive the contents of fixture files to be used in tests.  It is easier
  # to view, modify and visualize text used in tests when it is stored in files.
  # Also, it is much easier to reuse that text.
  def fixture_file(file_name)
    File.open(fixture_file_path(file_name), 'rb').read
  end

  def fixture_file_path(file_name)
    File.join(File.dirname(__FILE__), 'fixtures', file_name)
  end

  # This returns a lambda for use as the block argument of #stub. The lambda
  # returned calls #execute on the object being stubbed.
  def call_execute
    ->(stubbed_object) { stubbed_object.execute }
  end

  # Verifies that stubbed out requests have actually been requested.
  #
  # expectations - A variable number of RequestExpectation objects.
  def expect_requests(*expectations)
    yield
    expectations.
      flatten.
      each { |expectation| assert_requested(*expectation.to_args) }
  end
end
